/*----------------------------------------------------------------------

ColorSafe 1.0 Beta - http://www.woven-media.com/

This code has only been slightly changed and retains the original
Sun Microsytems, Inc. copyright notice.

Brent Allen Parrish <info@woven-media.com>

 * Copyright (c) 1995 - 2008 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Sun Microsystems nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
package com.woven_media.colorsafe;

import java.util.Vector;
import java.awt.Component;
import java.awt.Container;
import java.awt.FocusTraversalPolicy;

/**
 * NewFocusTraversalPolicy defines a new <code>java.awt.FocusTraversalPolicy</code>.
 * Based entirely on code examples from Sun Microsystems, Inc.. Class name was changed
 * from <code>MyNewFocusTraversalPolicy</code> to current class name.
 *
 * @author  Sun Microsystems, Inc.
 * @version %I%, %G%
 * @since   1.0
 * @see     java.awt.FocusTraversalPolicy
 */
public class NewFocusTraversalPolicy extends FocusTraversalPolicy
{
    Vector<Component> order;

    /**
     * Class constructor builds a <code>java.util.Vector&lt;java.awt.Component&gt;</code> collection
     * which defines the focus order for the components in this collection.
     *
     * @param order <code>java.util.Vector&lt;java.awt.Component&gt;</code> of components
     *              to add to the focus cycle.
     * @since       1.0
     */
    NewFocusTraversalPolicy(Vector<Component> order)
    {
        this.order = new Vector<Component>(order.size());
        this.order.addAll(order);
    }

    /**
     * Returns the component after the current component in this collection.
     *
     * @return <code>java.awt.Component</code> at specified index in this collection.
     * @since  1.0
     */
    public Component getComponentAfter(Container focusCycleRoot, Component aComponent)
    {
        int idx = (order.indexOf(aComponent) + 1) % order.size();
        return order.get(idx);
    }

    /**
     * Returns the component before the current component in this collection.
     *
     * @return <code>java.awt.Component</code> at the specified index in this collection.
     * @since  1.0
     */
    public Component getComponentBefore(Container focusCycleRoot, Component aComponent)
    {
        int idx = order.indexOf(aComponent) - 1;
        if (idx < 0)
        {
            idx = order.size() - 1;
        }
        return order.get(idx);
    }

    /**
     * Returns the FocusCycleRoot component.
     *
     * @return <code>java.awt.Component</code> representing this FocucCycleRoot component.
     * @since  1.0
     */
    public Component getDefaultComponent(Container focusCycleRoot)
    {
        return order.get(0);
    }

    /**
     * Returns the last <code>java.awt.Component</code> in this collection.
     *
     * @return <code>java.awt.Component</code> contained in this collection.
     * @since  1.0
     */
    public Component getLastComponent(Container focusCycleRoot)
    {
        return order.lastElement();
    }

    /**
     * Returns the first <code>java.awt.Component</code> in this collection.
     *
     * @return <code>java.awt.Component</code> contained in this collection.
     * @since  1.0
     */
    public Component getFirstComponent(Container focusCycleRoot)
    {
        return order.get(0);
    }

    /**************************************************
    USAGE EXAMPLE:

    NewFocusTraversalPolicy newPolicy;

    Vector<Component> order = new Vector<Component>(4);
    order.add(textfield1);
    order.add(textfield2);
    order.add(textfield3);
    order.add(table);
    newPolicy = new NewFocusTraversalPolicy(order);
    ***************************************************/

} // End class